@@ -263,13 +263,14 @@ find_hash (systemtap_session& s, const string& script)
 
 
 string
-find_tracequery_hash (systemtap_session& s, const string& header)
+find_tracequery_hash (systemtap_session& s, const vector<string>& headers)
 {
   hash h;
   get_base_hash(s, h);
 
-  // Add the tracepoint header to the computed hash
-  h.add_file(header);
+  // Add the tracepoint headers to the computed hash
+  for (size_t i = 0; i < headers.size(); ++i)
+    h.add_file(headers[i]);
 
   // Add any custom kbuild flags
   for (unsigned i = 0; i < s.kbuildflags.size(); i++)
