@@ -479,30 +479,30 @@ procfs_builder::build(systemtap_session & sess,
           // Make sure it doesn't start with '/'.
           if (end_pos == 0)
             throw semantic_error ("procfs path cannot start with a '/'",
-                                  location->tok);
+                                  location->components.front()->tok);
 
           component = path.substr(start_pos, end_pos - start_pos);
           // Make sure it isn't empty.
           if (component.size() == 0)
             throw semantic_error ("procfs path component cannot be empty",
-                                  location->tok);
+                                  location->components.front()->tok);
           // Make sure it isn't relative.
           else if (component == "." || component == "..")
-            throw semantic_error ("procfs path cannot be relative (and contain '.' or '..')", location->tok);
+            throw semantic_error ("procfs path cannot be relative (and contain '.' or '..')", location->components.front()->tok);
 
           start_pos = end_pos + 1;
         }
       component = path.substr(start_pos);
       // Make sure it doesn't end with '/'.
       if (component.size() == 0)
-        throw semantic_error ("procfs path cannot end with a '/'", location->tok);
+        throw semantic_error ("procfs path cannot end with a '/'", location->components.front()->tok);
       // Make sure it isn't relative.
       else if (component == "." || component == "..")
-        throw semantic_error ("procfs path cannot be relative (and contain '.' or '..')", location->tok);
+        throw semantic_error ("procfs path cannot be relative (and contain '.' or '..')", location->components.front()->tok);
     }
 
   if (!(has_read ^ has_write))
-    throw semantic_error ("need read/write component", location->tok);
+    throw semantic_error ("need read/write component", location->components.front()->tok);
 
   finished_results.push_back(new procfs_derived_probe(sess, base, location,
                                                       path, has_write));
