@@ -46,10 +46,19 @@ proc run_one_test {filename flags bits} {
     set sys_prog "[file dirname [file normalize $filename]]/sys.stp"
     set cmd "stap --skip-badvars -c $dir/${testname} ${sys_prog}"
     
+    # Extract additional C flags needed to compile
+    set add_flags ""
+    foreach i $flags {
+	if [regexp "^additional_flags=" $i] {
+	    regsub "^additional_flags=" $i "" tmp
+	    append add_flags " $tmp"
+	}
+    }
+
     # Extract the expected results
     # Use the preprocessor so we can ifdef tests in and out
     
-    set ccmd "gcc -E -C -P $filename"
+    set ccmd "gcc -E -C -P $add_flags $filename"
     # XXX: but note, this will expand all system headers too!
     catch {eval exec $ccmd} output
     
@@ -67,6 +76,12 @@ proc run_one_test {filename flags bits} {
 	    #regsub -all {\+} $line {\\+} line
 	    #regsub -all {\*} $line {\\*} line
 	    
+	    # Turn '[[[[' and ']]]]' into '(' and ')' respectively.
+	    # Because normally parens get quoted, this allows us to
+	    # have non-quoted parens.
+	    regsub -all {\[\[\[\[} $line {(} line
+	    regsub -all {\]\]\]\]} $line {)} line
+
 	    regsub -all NNNN $line {[\-0-9]+} line
 	    regsub -all XXXX $line {[x0-9a-fA-F]+} line
 	    
