@@ -22,6 +22,7 @@
 
 extern "C" {
 #include <fcntl.h>
+#include <grp.h>
 #include <pwd.h>
 #include <spawn.h>
 #include <stdio.h>
@@ -78,8 +79,8 @@ file_exists (const string &path)
 
 // Copy a file.  The copy is done via a temporary file and atomic
 // rename.
-int
-copy_file(const char *src, const char *dest)
+bool
+copy_file(const string& src, const string& dest, bool verbose)
 {
   int fd1, fd2;
   char buf[10240];
@@ -88,10 +89,13 @@ copy_file(const char *src, const char *dest)
   char *tmp_name;
   mode_t mask;
 
+  if (verbose)
+    clog << "Copying " << src << " to " << dest << endl;
+
   // Open the src file.
-  fd1 = open(src, O_RDONLY);
+  fd1 = open(src.c_str(), O_RDONLY);
   if (fd1 == -1)
-    return -1;
+    goto error;
 
   // Open the temporary output file.
   tmp = dest + string(".XXXXXX");
@@ -100,7 +104,7 @@ copy_file(const char *src, const char *dest)
   if (fd2 == -1)
     {
       close(fd1);
-      return -1;
+      goto error;
     }
 
   // Copy the src file to the temporary output file.
@@ -111,7 +115,7 @@ copy_file(const char *src, const char *dest)
 	  close(fd2);
 	  close(fd1);
 	  unlink(tmp_name);
-	  return -1;
+          goto error;
 	}
     }
   close(fd1);
@@ -126,18 +130,23 @@ copy_file(const char *src, const char *dest)
   if (close(fd2) == -1)
     {
       unlink(tmp_name);
-      return -1;
+      goto error;
     }
 
   // Rename the temporary output file to the destination file.
-  unlink(dest);
-  if (rename(tmp_name, dest) == -1)
+  unlink(dest.c_str());
+  if (rename(tmp_name, dest.c_str()) == -1)
     {
       unlink(tmp_name);
-      return -1;
+      goto error;
     }
 
-  return 0;
+  return true;
+
+error:
+  cerr << "Copy failed (\"" << src << "\" to \"" << dest << "\"): "
+       << strerror(errno) << endl;
+  return false;
 }
 
 
@@ -180,6 +189,36 @@ remove_file_or_dir (const char *name)
   return 0;
 }
 
+// Determine whether the current user is in the given group
+// by gid.
+bool
+in_group_id (gid_t target_gid)
+{
+  // According to the getgroups() man page, getgroups() may not
+  // return the effective gid, so try to match it first. */
+  if (target_gid == getegid())
+    return true;
+
+  // Get the list of the user's groups.
+  int ngids = getgroups(0, 0); // Returns the number to allocate.
+  if (ngids > 0) {
+    gid_t gidlist[ngids];
+    ngids = getgroups(ngids, gidlist);
+    for (int i = 0; i < ngids; i++) {
+      // If the user is a member of the target group, then we're done.
+      if (gidlist[i] == target_gid)
+	return true;
+    }
+  }
+  if (ngids < 0) {
+    cerr << "Unable to retrieve group list" << endl;
+    return false;
+  }
+
+  // The user is not a member of the target group
+  return false;
+}
+
 void
 tokenize(const string& str, vector<string>& tokens,
 	 const string& delimiters = " ")
@@ -205,7 +244,7 @@ tokenize(const string& str, vector<string>& tokens,
 // same policy as execvp().  A program name not containing a slash
 // will be searched along the $PATH.
 
-string find_executable(const string& name)
+string find_executable(const string& name, const string& env_path)
 {
   string retpath;
 
@@ -220,7 +259,7 @@ string find_executable(const string& name)
     }
   else // Nope, search $PATH.
     {
-      char *path = getenv("PATH");
+      char *path = getenv(env_path.c_str());
       if (path)
         {
           // Split PATH up.
@@ -335,7 +374,7 @@ stap_system(int verbose, const std::string& command)
 
   spawned_pid = 0;
 
-  if (verbose > 1) 
+  if (verbose > 1)
     clog << "Running " << command << endl;
 
   ret = posix_spawn(&spawned_pid, "/bin/sh", NULL, NULL, const_cast<char * const *>(argv), environ);
@@ -346,19 +385,19 @@ stap_system(int verbose, const std::string& command)
       if (ret == spawned_pid)
         {
           ret = WIFEXITED(status) ? WEXITSTATUS(status) : 128 + WTERMSIG(status);
-          if (verbose > 2) 
+          if (verbose > 2)
             clog << "Spawn waitpid result (0x" << ios::hex << status << ios::dec << "): " << ret << endl;
         }
       else
         {
-          if (verbose > 1) 
+          if (verbose > 1)
             clog << "Spawn waitpid error (" << ret << "): " << strerror(errno) << endl;
           ret = -1;
         }
     }
   else
     {
-      if (verbose > 1) 
+      if (verbose > 1)
         clog << "Spawn error (" << ret << "): " << strerror(ret) << endl;
       ret = -1;
     }
